/* -*-c-*-
 *
 * $Id: gfreduce-exp.h,v 1.2 2004/03/21 22:52:06 mdw Exp $
 *
 * Exponentiation operations for binary field reduction
 *
 * (c) 2004 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: gfreduce-exp.h,v $
 * Revision 1.2  2004/03/21 22:52:06  mdw
 * Merge and close elliptic curve branch.
 *
 * Revision 1.1.2.1  2004/03/21 22:39:46  mdw
 * Elliptic curves on binary fields work.
 *
 */

#ifndef CATACOMB_GFREDUCE_EXP_H
#define CATACOMB_GFREDUCE_EXP_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Exponentiation definitions ----------------------------------------*/

#define EXP_TYPE mp *

#define EXP_COPY(d, x) d = MP_COPY(x)
#define EXP_DROP(x) MP_DROP(x)

#define EXP_MUL(a, x) do {						\
  mp *t = gf_mul(spare, a, x);						\
  spare = a;								\
  a = gfreduce_do(gr, t, t);						\
} while (0)

#define EXP_SQR(a) do {							\
  mp *t = gf_sqr(spare, a);						\
  spare = a;								\
  a = gfreduce_do(gr, t, t);						\
} while (0)

#define EXP_FIX(x)

#define EXP_SETMUL(d, x, y) do {					\
  d = gf_mul(MP_NEW, x, y);						\
  d = gfreduce_do(gr, d, d);						\
} while (0)

#define EXP_SETSQR(d, x) do {						\
  d = gf_sqr(MP_NEW, x);						\
  d = gfreduce_do(gr, d, d);						\
} while (0)

#include "exp.h"

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
